@@ -33,12 +33,13 @@
 
 from kamaki.cli import command
 from kamaki.cli.command_tree import CommandTree
-from kamaki.cli.utils import print_dict, print_list, print_items
-from kamaki.cli.errors import raiseCLIError, CLISyntaxError
+from kamaki.cli.utils import print_dict, remove_from_items
+from kamaki.cli.errors import raiseCLIError, CLISyntaxError, CLIBaseUrlError
 from kamaki.clients.cyclades import CycladesClient, ClientError
 from kamaki.cli.argument import FlagArgument, ValueArgument, KeyValueArgument
 from kamaki.cli.argument import ProgressBarArgument, DateArgument, IntArgument
-from kamaki.cli.commands import _command_init, errors
+from kamaki.cli.commands import _command_init, errors, addLogSettings
+from kamaki.cli.commands import _optional_output_cmd, _optional_json
 
 from base64 import b64encode
 from os.path import exists
@@ -52,7 +53,7 @@ _commands = [server_cmds, flavor_cmds, network_cmds]
 
 about_authentication = '\nUser Authentication:\
     \n* to check authentication: /user authenticate\
-    \n* to set authentication token: /config set token <token>'
+    \n* to set authentication token: /config set cloud.<cloud>.token <token>'
 
 howto_personality = [
     'Defines a file to be injected to VMs personality.',
@@ -66,21 +67,36 @@ howto_personality = [
 
 class _init_cyclades(_command_init):
     @errors.generic.all
+    @addLogSettings
     def _run(self, service='compute'):
-        token = self.config.get(service, 'token')\
-            or self.config.get('global', 'token')
-        base_url = self.config.get(service, 'url')\
-            or self.config.get('global', 'url')
-        self.client = CycladesClient(base_url=base_url, token=token)
-        self._set_log_params()
-        self._update_max_threads()
+        if getattr(self, 'cloud', None):
+            base_url = self._custom_url(service)\
+                or self._custom_url('cyclades')
+            if base_url:
+                token = self._custom_token(service)\
+                    or self._custom_token('cyclades')\
+                    or self.config.get_cloud('token')
+                self.client = CycladesClient(
+                    base_url=base_url, token=token)
+                return
+        else:
+            self.cloud = 'default'
+        if getattr(self, 'auth_base', False):
+            cyclades_endpoints = self.auth_base.get_service_endpoints(
+                self._custom_type('cyclades') or 'compute',
+                self._custom_version('cyclades') or '')
+            base_url = cyclades_endpoints['publicURL']
+            token = self.auth_base.token
+            self.client = CycladesClient(base_url=base_url, token=token)
+        else:
+            raise CLIBaseUrlError(service='cyclades')
 
     def main(self):
         self._run()
 
 
 @command(server_cmds)
-class server_list(_init_cyclades):
+class server_list(_init_cyclades, _optional_json):
     """List Virtual Machines accessible by user"""
 
     __doc__ += about_authentication
@@ -93,39 +109,24 @@ class server_list(_init_cyclades):
         limit=IntArgument('limit number of listed VMs', ('-n', '--number')),
         more=FlagArgument(
             'output results in pages (-n to set items per page, default 10)',
-            '--more')
+            '--more'),
+        enum=FlagArgument('Enumerate results', '--enumerate')
     )
 
-    def _make_results_pretty(self, servers):
-        for server in servers:
-            addr_dict = {}
-            if 'attachments' in server:
-                for addr in server['attachments']['values']:
-                    ips = addr.pop('values', [])
-                    for ip in ips:
-                        addr['IPv%s' % ip['version']] = ip['addr']
-                    if 'firewallProfile' in addr:
-                        addr['firewall'] = addr.pop('firewallProfile')
-                    addr_dict[addr.pop('id')] = addr
-                server['attachments'] = addr_dict if addr_dict else None
-            if 'metadata' in server:
-                server['metadata'] = server['metadata']['values']
-
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.date
     def _run(self):
         servers = self.client.list_servers(self['detail'], self['since'])
-        if self['detail']:
-            self._make_results_pretty(servers)
+        if not (self['detail'] or self['json_output']):
+            remove_from_items(servers, 'links')
 
+        kwargs = dict(with_enumeration=self['enum'])
         if self['more']:
-            print_items(
-                servers,
-                page_size=self['limit'] if self['limit'] else 10)
-        else:
-            print_items(
-                servers[:self['limit'] if self['limit'] else len(servers)])
+            kwargs['page_size'] = self['limit'] if self['limit'] else 10
+        elif self['limit']:
+            servers = servers[:self['limit']]
+        self._print(servers, **kwargs)
 
     def main(self):
         super(self.__class__, self)._run()
@@ -133,7 +134,7 @@ class server_list(_init_cyclades):
 
 
 @command(server_cmds)
-class server_info(_init_cyclades):
+class server_info(_init_cyclades, _optional_json):
     """Detailed information on a Virtual Machine
     Contains:
     - name, id, status, create/update dates
@@ -142,28 +143,11 @@ class server_info(_init_cyclades):
     - hardware flavor and os image ids
     """
 
-    def _print(self, server):
-        addr_dict = {}
-        if 'attachments' in server:
-            atts = server.pop('attachments')
-            for addr in atts['values']:
-                ips = addr.pop('values', [])
-                for ip in ips:
-                    addr['IPv%s' % ip['version']] = ip['addr']
-                if 'firewallProfile' in addr:
-                    addr['firewall'] = addr.pop('firewallProfile')
-                addr_dict[addr.pop('id')] = addr
-            server['attachments'] = addr_dict if addr_dict else None
-        if 'metadata' in server:
-            server['metadata'] = server['metadata']['values']
-        print_dict(server, ident=1)
-
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        server = self.client.get_server_details(server_id)
-        self._print(server)
+        self._print(self.client.get_server_details(server_id), print_dict)
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -205,7 +189,7 @@ class PersonalityArgument(KeyValueArgument):
 
 
 @command(server_cmds)
-class server_create(_init_cyclades):
+class server_create(_init_cyclades, _optional_json):
     """Create a server (aka Virtual Machine)
     Parameters:
     - name: (single quoted text)
@@ -215,8 +199,7 @@ class server_create(_init_cyclades):
 
     arguments = dict(
         personality=PersonalityArgument(
-            ' /// '.join(howto_personality),
-            ('-p', '--personality'))
+            (80 * ' ').join(howto_personality), ('-p', '--personality'))
     )
 
     @errors.generic.all
@@ -224,12 +207,10 @@ class server_create(_init_cyclades):
     @errors.plankton.id
     @errors.cyclades.flavor_id
     def _run(self, name, flavor_id, image_id):
-        r = self.client.create_server(
-            name,
-            int(flavor_id),
-            image_id,
-            self['personality'])
-        print_dict(r)
+        self._print(
+            self.client.create_server(
+                name, int(flavor_id), image_id, self['personality']),
+            print_dict)
 
     def main(self, name, flavor_id, image_id):
         super(self.__class__, self)._run()
@@ -237,7 +218,7 @@ class server_create(_init_cyclades):
 
 
 @command(server_cmds)
-class server_rename(_init_cyclades):
+class server_rename(_init_cyclades, _optional_output_cmd):
     """Set/update a server (VM) name
     VM names are not unique, therefore multiple servers may share the same name
     """
@@ -246,7 +227,8 @@ class server_rename(_init_cyclades):
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id, new_name):
-        self.client.update_server_name(int(server_id), new_name)
+        self._optional_output(
+            self.client.update_server_name(int(server_id), new_name))
 
     def main(self, server_id, new_name):
         super(self.__class__, self)._run()
@@ -254,14 +236,14 @@ class server_rename(_init_cyclades):
 
 
 @command(server_cmds)
-class server_delete(_init_cyclades):
+class server_delete(_init_cyclades, _optional_output_cmd):
     """Delete a server (VM)"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-            self.client.delete_server(int(server_id))
+            self._optional_output(self.client.delete_server(int(server_id)))
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -269,7 +251,7 @@ class server_delete(_init_cyclades):
 
 
 @command(server_cmds)
-class server_reboot(_init_cyclades):
+class server_reboot(_init_cyclades, _optional_output_cmd):
     """Reboot a server (VM)"""
 
     arguments = dict(
@@ -280,7 +262,8 @@ class server_reboot(_init_cyclades):
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        self.client.reboot_server(int(server_id), self['hard'])
+        self._optional_output(
+            self.client.reboot_server(int(server_id), self['hard']))
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -288,14 +271,14 @@ class server_reboot(_init_cyclades):
 
 
 @command(server_cmds)
-class server_start(_init_cyclades):
+class server_start(_init_cyclades, _optional_output_cmd):
     """Start an existing server (VM)"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        self.client.start_server(int(server_id))
+        self._optional_output(self.client.start_server(int(server_id)))
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -303,14 +286,14 @@ class server_start(_init_cyclades):
 
 
 @command(server_cmds)
-class server_shutdown(_init_cyclades):
+class server_shutdown(_init_cyclades, _optional_output_cmd):
     """Shutdown an active server (VM)"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        self.client.shutdown_server(int(server_id))
+        self._optional_output(self.client.shutdown_server(int(server_id)))
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -318,7 +301,7 @@ class server_shutdown(_init_cyclades):
 
 
 @command(server_cmds)
-class server_console(_init_cyclades):
+class server_console(_init_cyclades, _optional_json):
     """Get a VNC console to access an existing server (VM)
     Console connection information provided (at least):
     - host: (url or address) a VNC host
@@ -330,8 +313,8 @@ class server_console(_init_cyclades):
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        r = self.client.get_server_console(int(server_id))
-        print_dict(r)
+        self._print(
+            self.client.get_server_console(int(server_id)), print_dict)
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -340,6 +323,11 @@ class server_console(_init_cyclades):
 
 @command(server_cmds)
 class server_firewall(_init_cyclades):
+    """Manage server (VM) firewall profiles for public networks"""
+
+
+@command(server_cmds)
+class server_firewall_set(_init_cyclades, _optional_output_cmd):
     """Set the server (VM) firewall profile on VMs public network
     Values for profile:
     - DISABLED: Shutdown firewall
@@ -352,9 +340,8 @@ class server_firewall(_init_cyclades):
     @errors.cyclades.server_id
     @errors.cyclades.firewall
     def _run(self, server_id, profile):
-        self.client.set_firewall_profile(
-            server_id=int(server_id),
-            profile=('%s' % profile).upper())
+        self._optional_output(self.client.set_firewall_profile(
+            server_id=int(server_id), profile=('%s' % profile).upper()))
 
     def main(self, server_id, profile):
         super(self.__class__, self)._run()
@@ -362,15 +349,14 @@ class server_firewall(_init_cyclades):
 
 
 @command(server_cmds)
-class server_addr(_init_cyclades):
-    """List the addresses of all network interfaces on a server (VM)"""
+class server_firewall_get(_init_cyclades):
+    """Get the server (VM) firewall profile for its public network"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        reply = self.client.list_server_nics(int(server_id))
-        print_list(reply, with_enumeration=len(reply) > 1)
+        print(self.client.get_firewall_profile(server_id))
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -378,18 +364,42 @@ class server_addr(_init_cyclades):
 
 
 @command(server_cmds)
-class server_meta(_init_cyclades):
-    """Get a server's metadatum
-    Metadata are formed as key:value pairs where key is used to retrieve them
-    """
+class server_addr(_init_cyclades, _optional_json):
+    """List the addresses of all network interfaces on a server (VM)"""
+
+    arguments = dict(
+        enum=FlagArgument('Enumerate results', '--enumerate')
+    )
+
+    @errors.generic.all
+    @errors.cyclades.connection
+    @errors.cyclades.server_id
+    def _run(self, server_id):
+        reply = self.client.list_server_nics(int(server_id))
+        self._print(
+            reply, with_enumeration=self['enum'] and len(reply) > 1)
+
+    def main(self, server_id):
+        super(self.__class__, self)._run()
+        self._run(server_id=server_id)
+
+
+@command(server_cmds)
+class server_metadata(_init_cyclades):
+    """Manage Server metadata (key:value pairs of server attributes)"""
+
+
+@command(server_cmds)
+class server_metadata_list(_init_cyclades, _optional_json):
+    """Get server metadata"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     @errors.cyclades.metadata
     def _run(self, server_id, key=''):
-        r = self.client.get_server_metadata(int(server_id), key)
-        print_dict(r)
+        self._print(
+            self.client.get_server_metadata(int(server_id), key), print_dict)
 
     def main(self, server_id, key=''):
         super(self.__class__, self)._run()
@@ -397,26 +407,42 @@ class server_meta(_init_cyclades):
 
 
 @command(server_cmds)
-class server_setmeta(_init_cyclades):
-    """set server (VM) metadata
-    Metadata are formed as key:value pairs, both needed to set one
+class server_metadata_set(_init_cyclades, _optional_json):
+    """Set / update server(VM) metadata
+    Metadata should be given in key/value pairs in key=value format
+    For example: /server metadata set <server id> key1=value1 key2=value2
+    Old, unreferenced metadata will remain intact
     """
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
-    def _run(self, server_id, key, val):
-        metadata = {key: val}
-        r = self.client.update_server_metadata(int(server_id), **metadata)
-        print_dict(r)
+    def _run(self, server_id, keyvals):
+        assert keyvals, 'Please, add some metadata ( key=value)'
+        metadata = dict()
+        for keyval in keyvals:
+            k, sep, v = keyval.partition('=')
+            if sep and k:
+                metadata[k] = v
+            else:
+                raiseCLIError(
+                    'Invalid piece of metadata %s' % keyval,
+                    importance=2, details=[
+                        'Correct metadata format: key=val',
+                        'For example:',
+                        '/server metadata set <server id>'
+                        'key1=value1 key2=value2'])
+        self._print(
+            self.client.update_server_metadata(int(server_id), **metadata),
+            print_dict)
 
-    def main(self, server_id, key, val):
+    def main(self, server_id, *key_equals_val):
         super(self.__class__, self)._run()
-        self._run(server_id=server_id, key=key, val=val)
+        self._run(server_id=server_id, keyvals=key_equals_val)
 
 
 @command(server_cmds)
-class server_delmeta(_init_cyclades):
+class server_metadata_delete(_init_cyclades, _optional_output_cmd):
     """Delete server (VM) metadata"""
 
     @errors.generic.all
@@ -424,7 +450,8 @@ class server_delmeta(_init_cyclades):
     @errors.cyclades.server_id
     @errors.cyclades.metadata
     def _run(self, server_id, key):
-        self.client.delete_server_metadata(int(server_id), key)
+        self._optional_output(
+            self.client.delete_server_metadata(int(server_id), key))
 
     def main(self, server_id, key):
         super(self.__class__, self)._run()
@@ -432,15 +459,14 @@ class server_delmeta(_init_cyclades):
 
 
 @command(server_cmds)
-class server_stats(_init_cyclades):
+class server_stats(_init_cyclades, _optional_json):
     """Get server (VM) statistics"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.server_id
     def _run(self, server_id):
-        r = self.client.get_server_stats(int(server_id))
-        print_dict(r, exclude=('serverRef',))
+        self._print(self.client.get_server_stats(int(server_id)), print_dict)
 
     def main(self, server_id):
         super(self.__class__, self)._run()
@@ -487,7 +513,7 @@ class server_wait(_init_cyclades):
 
 
 @command(flavor_cmds)
-class flavor_list(_init_cyclades):
+class flavor_list(_init_cyclades, _optional_json):
     """List available hardware flavors"""
 
     arguments = dict(
@@ -495,15 +521,22 @@ class flavor_list(_init_cyclades):
         limit=IntArgument('limit # of listed flavors', ('-n', '--number')),
         more=FlagArgument(
             'output results in pages (-n to set items per page, default 10)',
-            '--more')
+            '--more'),
+        enum=FlagArgument('Enumerate results', '--enumerate')
     )
 
     @errors.generic.all
     @errors.cyclades.connection
     def _run(self):
         flavors = self.client.list_flavors(self['detail'])
+        if not (self['detail'] or self['json_output']):
+            remove_from_items(flavors, 'links')
         pg_size = 10 if self['more'] and not self['limit'] else self['limit']
-        print_items(flavors, with_redundancy=self['detail'], page_size=pg_size)
+        self._print(
+            flavors,
+            with_redundancy=self['detail'],
+            page_size=pg_size,
+            with_enumeration=self['enum'])
 
     def main(self):
         super(self.__class__, self)._run()
@@ -511,7 +544,7 @@ class flavor_list(_init_cyclades):
 
 
 @command(flavor_cmds)
-class flavor_info(_init_cyclades):
+class flavor_info(_init_cyclades, _optional_json):
     """Detailed information on a hardware flavor
     To get a list of available flavors and flavor ids, try /flavor list
     """
@@ -520,8 +553,8 @@ class flavor_info(_init_cyclades):
     @errors.cyclades.connection
     @errors.cyclades.flavor_id
     def _run(self, flavor_id):
-        flavor = self.client.get_flavor_details(int(flavor_id))
-        print_dict(flavor)
+        self._print(
+            self.client.get_flavor_details(int(flavor_id)), print_dict)
 
     def main(self, flavor_id):
         super(self.__class__, self)._run()
@@ -529,25 +562,17 @@ class flavor_info(_init_cyclades):
 
 
 @command(network_cmds)
-class network_info(_init_cyclades):
+class network_info(_init_cyclades, _optional_json):
     """Detailed information on a network
     To get a list of available networks and network ids, try /network list
     """
 
-    @classmethod
-    def _make_result_pretty(self, net):
-        if 'attachments' in net:
-            att = net['attachments']['values']
-            count = len(att)
-            net['attachments'] = att if count else None
-
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.network_id
     def _run(self, network_id):
         network = self.client.get_network_details(int(network_id))
-        self._make_result_pretty(network)
-        print_dict(network, exclude=('id'))
+        self._print(network, print_dict, exclude=('id'))
 
     def main(self, network_id):
         super(self.__class__, self)._run()
@@ -555,7 +580,7 @@ class network_info(_init_cyclades):
 
 
 @command(network_cmds)
-class network_list(_init_cyclades):
+class network_list(_init_cyclades, _optional_json):
     """List networks"""
 
     arguments = dict(
@@ -563,25 +588,22 @@ class network_list(_init_cyclades):
         limit=IntArgument('limit # of listed networks', ('-n', '--number')),
         more=FlagArgument(
             'output results in pages (-n to set items per page, default 10)',
-            '--more')
+            '--more'),
+        enum=FlagArgument('Enumerate results', '--enumerate')
     )
 
-    def _make_results_pretty(self, nets):
-        for net in nets:
-            network_info._make_result_pretty(net)
-
     @errors.generic.all
     @errors.cyclades.connection
     def _run(self):
         networks = self.client.list_networks(self['detail'])
-        if self['detail']:
-            self._make_results_pretty(networks)
+        if not (self['detail'] or self['json_output']):
+            remove_from_items(networks, 'links')
+        kwargs = dict(with_enumeration=self['enum'])
         if self['more']:
-            print_items(networks, page_size=self['limit'] or 10)
+            kwargs['page_size'] = self['limit'] or 10
         elif self['limit']:
-            print_items(networks[:self['limit']])
-        else:
-            print_items(networks)
+            networks = networks[:self['limit']]
+        self._print(networks, **kwargs)
 
     def main(self):
         super(self.__class__, self)._run()
@@ -589,7 +611,7 @@ class network_list(_init_cyclades):
 
 
 @command(network_cmds)
-class network_create(_init_cyclades):
+class network_create(_init_cyclades, _optional_json):
     """Create an (unconnected) network"""
 
     arguments = dict(
@@ -607,13 +629,12 @@ class network_create(_init_cyclades):
     @errors.cyclades.connection
     @errors.cyclades.network_max
     def _run(self, name):
-        r = self.client.create_network(
+        self._print(self.client.create_network(
             name,
             cidr=self['cidr'],
             gateway=self['gateway'],
             dhcp=self['dhcp'],
-            type=self['type'])
-        print_items([r])
+            type=self['type']), print_dict)
 
     def main(self, name):
         super(self.__class__, self)._run()
@@ -621,14 +642,15 @@ class network_create(_init_cyclades):
 
 
 @command(network_cmds)
-class network_rename(_init_cyclades):
+class network_rename(_init_cyclades, _optional_output_cmd):
     """Set the name of a network"""
 
     @errors.generic.all
     @errors.cyclades.connection
     @errors.cyclades.network_id
     def _run(self, network_id, new_name):
-        self.client.update_network_name(int(network_id), new_name)
+        self._optional_output(
+                self.client.update_network_name(int(network_id), new_name))
 
     def main(self, network_id, new_name):
         super(self.__class__, self)._run()
@@ -636,7 +658,7 @@ class network_rename(_init_cyclades):
 
 
 @command(network_cmds)
-class network_delete(_init_cyclades):
+class network_delete(_init_cyclades, _optional_output_cmd):
     """Delete a network"""
 
     @errors.generic.all
@@ -644,7 +666,7 @@ class network_delete(_init_cyclades):
     @errors.cyclades.network_id
     @errors.cyclades.network_in_use
     def _run(self, network_id):
-        self.client.delete_network(int(network_id))
+        self._optional_output(self.client.delete_network(int(network_id)))
 
     def main(self, network_id):
         super(self.__class__, self)._run()
@@ -652,7 +674,7 @@ class network_delete(_init_cyclades):
 
 
 @command(network_cmds)
-class network_connect(_init_cyclades):
+class network_connect(_init_cyclades, _optional_output_cmd):
     """Connect a server to a network"""
 
     @errors.generic.all
@@ -660,7 +682,8 @@ class network_connect(_init_cyclades):
     @errors.cyclades.server_id
     @errors.cyclades.network_id
     def _run(self, server_id, network_id):
-        self.client.connect_server(int(server_id), int(network_id))
+        self._optional_output(
+                self.client.connect_server(int(server_id), int(network_id)))
 
     def main(self, server_id, network_id):
         super(self.__class__, self)._run()
@@ -683,12 +706,14 @@ class network_disconnect(_init_cyclades):
     @errors.cyclades.server_id
     @errors.cyclades.nic_id
     def _run(self, nic_id, server_id):
-        if not self.client.disconnect_server(server_id, nic_id):
+        num_of_disconnected = self.client.disconnect_server(server_id, nic_id)
+        if not num_of_disconnected:
             raise ClientError(
                 'Network Interface %s not found on server %s' % (
                     nic_id,
                     server_id),
                 status=404)
+        print('Disconnected %s connections' % num_of_disconnected)
 
     def main(self, nic_id):
         super(self.__class__, self)._run()
