@@ -1,7 +1,7 @@
-__version__ = "0.8.6"
-__version_info__ = ['0', '8', '6']
+__version__ = "0.9"
+__version_info__ = ['0', '9']
 __version_vcs_info__ = {'branch': 'master',
- 'revid': '6ca5607',
- 'revno': 1295,
+ 'revid': 'c522b8e',
+ 'revno': 1505,
  'toplevel': '/home/saxtouri/src/kamaki/kamaki'}
 __version_user_info__ = "saxtouri@terminus"
